#!/bin/bash
#

# build_epics.sh <product_directory> <e6|e7|e9> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e6|e7|e9> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}
sourcetar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=epics
origpkgver=v3_15_2
pkgver=${origpkgver}
ssibuildshims_version=v0_08_06
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=base-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Unable to untar source."
cd ${pkgtarname}
export EPICS_HOST_ARCH=`./startup/EpicsHostArch.pl`
export INSTALL_DIR=${pkgdir}
mkdir -p $INSTALL_DIR
cp -a ${patchdir}/* ${sourcedir}/${pkgtarname}

make -j ${ncore} || die "ERROR: make failed."

mv ${sourcedir}/${pkgtarname}/documentation ${sourcedir}/../doc
mv ${sourcedir}/${pkgtarname}/startup ${sourcedir}/../scripts

set +x

# Clean up.
cd "${pkgdir}"

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "epics is installed at ${EPICS_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
